#' Correction Workflow for Multiple Choice Exams
#' 
#' This function will do the whole correction workflow for MC exams: Read
#' in MC-files, apply the grading scale, export the grades in edoz format and
#' knit the corresponding information into a tex-file (used later on for the 
#' overview PDF).
#'
#' @param MC.answerfile File name of export file of MC-software.
#' @param MC.answerfile.sep Separator used in \code{MC.answerfile}.
#' @param MC.keyfile File name of file containing solutions.
#' @param MC.keyfile.type Type of \code{MC.keyfile}. Either \code{"TF"} for 
#' TRUE/FALSE questions or \code{"AB"} for questions where the correct options 
#' between e.g. a) - d) has to be found (single choice questions).
#' @param MC.keyfile.sep Separator used in \code{MC.keyfile}.
#' @param MC.prefix.prelim Prefix which is used for columns containing ticked boxes
#' in \code{MC.answerfile}. Typically \code{"ticked"}.
#' @param edozFile File name of edoz file.
#' @param correct Number of points for correct answer.
#' @param wrong Number of points for wrong answer.
#' @param type Type of MC questions. Either \code{"SC"} (single choice) or 
#' \code{"kprim"} (k-prim)
#' @param gradeFunction.name Name of the grading function, e.g., \code{"gradesPiecewiseLinear"}.
#' @param gradeFunction.settings A list containting the options for \code{gradeFunction.name}.
#' @param number.replace.source Vector of Legi-numbers which will be replaced.
#' @param number.replace.dest Destination Legi-numbers of the numbers in 
#' \code{number.replace.source}.
#' @param rdataFile If not \code{NULL}, the relevant R objects will be stored in this file.
#' @param subtask.label Only used for student overview page: Either \code{"character"}
#' or \code{"numeric"}. For \code{"character"} the sub-tasks will be labeled
#' with lowercase letters, for \code{"numeric"} the sub-tasks will be enumerated.
#' @param minPoints Lower end when \emph{printing} grading scale.
#' @param maxPoints Upper end when \emph{printing} grading scale.
#' @param stepsize Stepsize used when \emph{printing} grading scale.
#'
#' @return knitted results
#' @export

process_corrections_MC <- function(MC.answerfile,
                                   MC.answerfile.sep,
                                   MC.keyfile,
                                   MC.keyfile.type,
                                   MC.keyfile.sep,
                                   MC.prefix.prelim,
                                   edozFile,
                                   correct,
                                   wrong,
                                   type,
                                   gradeFunction.name,
                                   gradeFunction.settings,
                                   number.replace.source,
                                   number.replace.dest,
                                   rdataFile,
                                   subtask.label,
                                   minPoints,
                                   maxPoints,
                                   stepsize) {
  #######################
  ## Read in all files ##
  #######################
  
  ## Convert MC.prefix to valid column naming scheme
  MC.prefix <- make.names(MC.prefix.prelim)
  
  ## Read data from edoz exported file
  edozList <- readEdoz(file = edozFile)
  
  ## Read MC answers
  MC.answers <- readMC.answers(file = MC.answerfile, sep = MC.answerfile.sep)
  
  ## Replace numbers if needed
  if(!is.null(number.replace.source)) {
    MC.answers <- replaceNumber(MC.answers = MC.answers,
                                edozList = edozList,
                                number.source = number.replace.source,
                                number.dest   = number.replace.dest
      )
  }
  
  ## Read MC key (solutions)
  MC.key <- readMC.key(file = MC.keyfile, type = MC.keyfile.type,
                       sep = MC.keyfile.sep)
  
  ## Consistency check MC.key and MC.answers
  check.answers.key(MC.key = MC.key, MC.answers = MC.answers, 
                    MC.prefix  = MC.prefix)
  
  #############################
  ## Apply correction scheme ##
  #############################
  
  ## Compare MC.answers with MC.key:
  ## Output is TRUE if correct box was ticked, otherwise FALSE
  correct.out <- correctMC(answers = MC.answers, solutions = MC.key,
                           MC.prefix = MC.prefix)
  MC.corrections <- correct.out$MC.corrections
  MC.crit.cases  <- correct.out$MC.crit.cases
  ## TODO: Make sure that MC.crit.cases is shown to end-user
  
  ## Give points based on corrected answers:
  ## Map TRUE / FALSE from above to points, grading scale *not* applied yet
  MC.results <- givePoints(corrections = MC.corrections, MC.key = MC.key, 
                           correct = correct, wrong = wrong, type = type)
  
  #########################
  ## Apply grading scale ##
  #########################
  
  ## Transform points into grade by applying grading function
  grade.matrix <- applyGrading(pointsTotal = MC.results$pointsTotal,
                               gradeFn     = get(gradeFunction.name),
                               settings    = gradeFunction.settings)
  
  ## Append grades to MC.results
  MC.results[,"rawGrade"]    <- grade.matrix[,"rawGrade"]
  MC.results[,"grade"]       <- grade.matrix[,"grade"]
  MC.results[,"roundEffect"] <- MC.results[,"grade"] - MC.results[, "rawGrade"]
  
  
  ###############################
  ## Export grades to edozFile ##
  ###############################
  
  edoz.out <- exportGrades(edozList = edozList, results = MC.results,
                           file = edozFile)
  
  ## Store and export grades to rda-file (if requested)
  if (!is.null(rdataFile))
    save(MC.results, gradeFunction.name, gradeFunction.settings,
         file = rdataFile)
  
  ############################
  ## Knit the overview file ##
  ############################

  dir.create("childs", showWarnings = FALSE) ## TODO: More elegant solution???
  
  in_file <- system.file("Rnw-files", "output-file-MC.Rnw", package = "sfsGrading")
  text_input <- paste0(readLines(in_file), collapse = "\n")

  knit(text = text_input, output = "childs/output-file-MC.tex", tangle = FALSE, envir = environment())
}