#' Correction Workflow for Moodle Exams
#' 
#' This function will do the whole correction workflow for Moodle exams: Read
#' in Moodle-file, apply the grading scale, export the grades in edoz format and
#' knit the corresponding information into a tex-file (used later on for the 
#' overview PDF).
#'
#' @param Moodle.pointsFile File name of Moodle export file.
#' @param Moodle.pointsFile.sep Separator used in Moodle export file.
#' @param edozFile File name of edoz file.
#' @param gradeFunction.name Name of the grading function, e.g., \code{"gradesPiecewiseLinear"}.
#' @param gradeFunction.settings A list containting the options for \code{gradeFunction.name}.
#' @param rdataFile If not \code{NULL}, the relevant R objects will be stored in this file.
#' @param minPoints Lower end when \emph{printing} grading scale.
#' @param maxPoints Upper end when \emph{printing} grading scale.
#' @param stepsize Stepsize used when \emph{printing} grading scale.
#'
#' @return knitted results
#' @export
process_corrections_Moodle <- function(Moodle.pointsFile,
                                       Moodle.pointsFile.sep,
                                       edozFile,
                                       gradeFunction.name,
                                       gradeFunction.settings,
                                       rdataFile,
                                       # from here: only needed in Rnw chunks
                                       minPoints,
                                       maxPoints,
                                       stepsize) {
  
  
  #######################
  ## Read in all files ##
  #######################
  
  ## Read data from edoz exported file
  edozList <- readEdoz(file = edozFile)
  
  ## Read Moodle points
  Moodle.input  <- readMoodle(file = Moodle.pointsFile, sep = Moodle.pointsFile.sep)
  Moodle.points <- Moodle.input$moodleFile
  maxPoints.q   <- Moodle.input$max.points
  
  #########################
  ## Apply grading scale ##
  #########################
  
  ## Transform points into grade by applying grading function
  grade.matrix <- applyGrading(pointsTotal = Moodle.points$pointsTotal, 
                               gradeFn     = get(gradeFunction.name),
                               settings    = gradeFunction.settings)
  
  ## Append grades to Moodle.points
  Moodle.points[,"rawGrade"]    <- grade.matrix[,"rawGrade"]
  Moodle.points[,"grade"]       <- grade.matrix[,"grade"]
  Moodle.points[,"roundEffect"] <- Moodle.points[,"grade"] - Moodle.points[,"rawGrade"]
  
  
  ###############################
  ## Export grades to edozFile ##
  ###############################
  
  edoz.out <- exportGrades(edozList = edozList, results = Moodle.points, 
                           file = edozFile)
  
  ## Store and export grades to rda-file (if requested)
  if(!is.null(rdataFile))
    save(Moodle.points, 
         gradeFunction.name, 
         gradeFunction.settings,
         file = rdataFile)
  
  ############################
  ## Knit the overview file ##
  ############################

  dir.create("childs", showWarnings = FALSE) ## TODO: More elegant solution???
    
  in_file <- system.file("Rnw-files", "output-file-Moodle.Rnw", package = "sfsGrading")
  text_input <- paste0(readLines(in_file), collapse = "\n")
  
  knit(text = text_input, output = "childs/output-file-Moodle.tex", envir = environment())
}
